import emot
import contractions

from autocorrect import Speller

from nltk.stem import PorterStemmer
from nltk.stem import LancasterStemmer
from nltk.stem import WordNetLemmatizer
from nltk.stem.snowball import SnowballStemmer


class TextNormalization:

    def __init__(self, stemmer='PorterStemmer', language='en'):
        self.stemmer = stemmer
        self.lemstem = self.get_lemstem
        self.checker = Speller(lang=language)

    @property
    def get_lemstem(self):
        if self.stemmer == "WordNetLemmatizer":
            return WordNetLemmatizer()
        elif self.stemmer == "LancasterStemmer":
            return LancasterStemmer("english")
        elif self.stemmer == "SnowballStemmer":
            return SnowballStemmer("english", ignore_stopwords=True)
        else:
            return PorterStemmer()

    def text_correction(self, text):
        return self.checker(text)

    def word_stemming(self, word):
        if self.stemmer == 'WordNetLemmatizer':
            return self.lemstem.lemmatize(word)
        else:
            return self.lemstem.stem(word)

    @staticmethod
    def text_demojis(text, how_replace="mean"):
        emojis = emot.emoji(text)
        if emojis['flag']:
            for index in range(len(emojis["value"])):
                if how_replace == 'mean':
                    source = emojis['value'][index]
                    target = emojis['mean'][index].split(':')[1]
                    text = text.replace(source, target)
                else:
                    text = text.replace(emojis['value'][index], "")

        return text

    @staticmethod
    def text_demoticons(text, how_replace="mean"):
        emoticons = emot.emoticons(text)
        if emoticons['flag']:
            for index in range(len(emoticons["value"])):
                if how_replace == 'mean':
                    source = emoticons['value'][index]
                    target = emoticons['mean'][index]
                    text = text.replace(source, target)
                else:
                    text = text.replace(emoticons['value'][index], "")

        return text

    @staticmethod
    def text_decontraction(text):
        return contractions.fix(text)
